NEWDESeq2 <- function(x, conditions, libTypes, treated, control, the.file, pvaladj, Project){

  require(DESeq2)

  if(Sys.info()[[1]]=="Windows"){
    a=paste(getwd(),"\\RNASeqGUI_Projects\\",Project,"\\Plots\\",sep="")
    the.file2 = strsplit(the.file,"\\\\")
    the.file2 = the.file2[[1]][length(the.file2[[1]])]  #estract the namefile
    sys.sep="\\"
  }else{ #Linux
    a=paste(getwd(),"/RNASeqGUI_Projects/",Project,"/Results/",sep="")
    the.file2 = strsplit(the.file,"/")
    the.file2 = the.file2[[1]][length(the.file2[[1]])]  #estract the namefile
    sys.sep="/"
  }
   
  db <- InitDb(db.name=paste(the.file2,'deseq2_db',sep="_"), db.path=file.path("RNASeqGUI_Projects",Project,"Logs","cache"))
  
  SaveInCache(db, the.file, "the_file_key")
  SaveInCache(db, Project, "project_key")
  SaveInCache(db, x, "maindataframe_key")
  SaveInCache(db, conditions, "conditions_key")
  SaveInCache(db, libTypes, "libtypes_key")
  SaveInCache(db, treated, "treated_key")
  SaveInCache(db, control, "control_key")
  
  pvaladj = as.numeric(pvaladj)
  countData <- x
  
  SaveInCache(db, pvaladj, "pvaladj_key")
  SaveInCache(db, countData, "countdata_key")

  print("You loaded this count file: ")
  print(head(countData))

  colData = data.frame(row.names=colnames(countData), condition=conditions, libType=libTypes)
  print(colData)
  SaveInCache(db, colData, "coldata_key")
  
  dds <- DESeqDataSetFromMatrix(countData = countData, colData = colData, design = ~ condition)

  SaveInCache(db, dds, "dds_key")

  colData(dds)$condition <- factor(colData(dds)$condition, levels=c(control,treated))
  SaveInCache(db, dds, "ddscond_key")
  ######################################## ANALISYS ###################################

  dds <- DESeq2::DESeq(dds)
  SaveInCache(db, dds, "ddsdes2_key")
  
  res <- results(dds)
  
  SaveInCache(db, res, "res_key")

  print("Firts five lines of the results.")
  print( head(res) )
  message=paste("Results of DESeq2 have been saved in the Results folder of ", Project," project!", sep="")
  print(message)
  
  res$id <- rownames(res)
  res<-(res[,c(7,1,2,3,4,5,6)])

  outputName=paste(the.file2,"_results_DESeq2.txt", sep="")
  b=paste(a,outputName,sep=sys.sep)
  write.table( as.data.frame(res), file = b, quote=FALSE, sep="\t", row.names=FALSE) 

  outputName=paste(the.file2,"_results_DESeq2.tsv", sep="")
  b=paste(a,outputName,sep=sys.sep)
  write.table( as.data.frame(res), file = b, quote=FALSE, sep="\t", row.names=FALSE)   

  list_DE_DESEQ2 = subset(as.data.frame(res), padj < pvaladj) # select significant genes
  SaveInCache(db, list_DE_DESEQ2, "listdedeseq2_key")
  
  outputName=paste(the.file2,"_padj=",pvaladj,"_DE_genes_DESeq2.txt", sep="")
  b=paste(a,outputName,sep=sys.sep)
  write.table(list_DE_DESEQ2, file = b , quote=FALSE, sep="\t", row.names=FALSE)  

  outputName=paste(the.file2,"_padj=",pvaladj,"_DE_genes_DESeq2.tsv", sep="")
  b=paste(a,outputName,sep=sys.sep)
  write.table(list_DE_DESEQ2, file = b , quote=FALSE, sep="\t", row.names=FALSE)  
 
  #PLOTS 

  a=paste(getwd(),"RNASeqGUI_Projects",Project,"Plots",sep=sys.sep)
  outputName=paste(the.file2,"_Dispersion_Local_DESeq2.pdf", sep="")
  b=paste(a,outputName,sep=sys.sep)
  ddsLocal <- estimateDispersions(dds, fitType="local")
  SaveInCache(db, ddsLocal, "ddsLocal_key")
  DESeq2::plotDispEsts(ddsLocal)

  dev.print(device = pdf, file=b)
  dev.off()
  outputName=paste(the.file2,"_Dispersion_Mean_DESeq2.pdf", sep="")
  b=paste(a,outputName,sep=sys.sep)
  ddsMean <- estimateDispersions(dds, fitType="mean")
  SaveInCache(db, ddsMean, "ddsmean_key")
  DESeq2::plotDispEsts(ddsMean)

  dev.print(device = pdf, file=b)
  dev.off()
  outputName=paste(the.file2,"_Dispersion_DESeq2.pdf", sep="")
  b=paste(a,outputName,sep=sys.sep)
  DESeq2::plotDispEsts(dds)

  dev.print(device = pdf, file=b)
  dev.off()

  list_rest_of_genes_DESEQ2 = subset(as.data.frame(res), (padj >= pvaladj) | (baseMean == 0) ) # select rest of genes
  list_UP_DESEQ2 = subset(list_DE_DESEQ2, list_DE_DESEQ2$log2FoldChange > 0)
  list_DOWN_DESEQ2 = subset(list_DE_DESEQ2, list_DE_DESEQ2$log2FoldChange < 0)
  num_up   = dim(list_UP_DESEQ2)[1]
  num_down = dim(list_DOWN_DESEQ2)[1]
  rest     = dim(list_rest_of_genes_DESEQ2)[1]
  slices <- c(num_up, num_down, rest)
  lbls <- c(paste(num_up,' UP Genes',sep=''), paste(num_down,' DOWN Genes',sep=''), paste(rest,' remaining Genes',sep=''))
  pie3D(slices,labels=lbls,explode=0.1, main='Pie Chart of DE Genes')
  
  if(Sys.info()[[1]]=="Windows"){
  #write into the project report
    report=paste("RNASeqGUI_Projects\\",Project,"\\Logs\\report.Rmd",sep="")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message2 <- paste(" * In the *DESeq2 Interface*, you clicked the **Run DESeq2** button at `", Sys.time(),"` and the ",outputName," file has been saved in the `", Project,"\\Results` folder.", sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message2 <- paste("You chose the following count file: `",the.file2,"`, padj: `",pvaladj,"`, Project: `",Project,"`, treated='",treated,"', control='",control,"', ",sep="\n")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "\n")    
    message1 <- paste("Factors= c(")
    write(message1, file = report,ncolumns = if(is.character(message1)) 1 else 5,append = TRUE, sep = "")   
    message2 <- paste("'",conditions[1:(length(conditions)-1)],"',",sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "")    
    message3 <- paste("'",conditions[length(conditions)],"'",sep="")
    write(message3, file = report,ncolumns = if(is.character(message3)) 1 else 5,append = TRUE, sep = "")    
    message4 <- paste("), ")
    write(message4, file = report,ncolumns = if(is.character(message4)) 1 else 5,append = TRUE, sep = "")    
    message1 <- paste("LibTypes= c(")
    write(message1, file = report,ncolumns = if(is.character(message1)) 1 else 5,append = TRUE, sep = "")    
    message2 <- paste("'",libTypes[1:(length(libTypes)-1)],"',",sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "")    
    message3 <- paste("'",libTypes[length(libTypes)],"'",sep="")
    write(message3, file = report,ncolumns = if(is.character(message3)) 1 else 5,append = TRUE, sep = "")    
    message4 <- paste("). ")
    write(message4, file = report,ncolumns = if(is.character(message4)) 1 else 5,append = TRUE, sep = "")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" This R code has been run:",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" ```{r} ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("require(DESeq2)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("the.file2='",the.file2,"'",sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("db <- InitDb(db.name=paste(the.file2,'deseq2_db',sep='_'), db.path='cache')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("x <- LoadCachedObject(db, 'maindataframe_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("the.file <- LoadCachedObject(db, 'the_file_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("Project <- LoadCachedObject(db, 'project_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("conditions <- LoadCachedObject(db, 'conditions_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("libtypes <- LoadCachedObject(db, 'libtypes_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("treated <- LoadCachedObject(db, 'treated_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("control <- LoadCachedObject(db, 'control_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("pvaladj <- LoadCachedObject(db, 'pvaladj_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("countData <- LoadCachedObject(db, 'countdata_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("  print('You loaded this count file: ')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("print(head(countData)) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("#colData = data.frame(row.names=colnames(countData), condition=conditions, libType=libTypes) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("colData <- LoadCachedObject(db, 'coldata_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("#dds <- DESeqDataSetFromMatrix(countData = countData, colData = colData, design = ~ condition) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'dds_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#colData(dds)$condition <- factor(colData(dds)$condition, levels=c(control,treated)) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'ddscond_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("######################################## ANALISYS ################################### ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#dds <- DESeq2::DESeq(dds) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'ddsdes2_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#res <- results(dds) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("res <- LoadCachedObject(db, 'res_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("print('Firts five lines of the results.') ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("print( head(res) ) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#list_DE_DESEQ2 = subset(as.data.frame(res), padj < pvaladj) # select significant genes",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("list_DE_DESEQ2 <- LoadCachedObject(db, 'listdedeseq2_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#PLOTS ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#ddsLocal <- estimateDispersions(dds, fitType='local')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n") 
    message5 <- paste("ddsLocal <- LoadCachedObject(db, 'ddsLocal_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("DESeq2::plotDispEsts(ddsLocal)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#ddsMean <- estimateDispersions(dds, fitType='mean')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("ddsMean <- LoadCachedObject(db, 'ddsmean_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("DESeq2::plotDispEsts(ddsMean)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("DESeq2::plotDispEsts(dds)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message8 <- paste(" ``` ",sep="\n")
    write(message8, file = report,ncolumns = if(is.character(message8)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" _______________________________________________________________________ ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
   

  }else{ #Linux

    #write into the project report
    report=paste("RNASeqGUI_Projects/",Project,"/Logs/report.Rmd",sep="")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message2 <- paste(" * In the *DESeq2 Interface*, you clicked the **Run DESeq2** button at `", Sys.time(),"` and the ",outputName," file has been saved in the `", Project,"/Results` folder.", sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message2 <- paste("You chose the following count file: `",the.file2,"`, padj: `",pvaladj,"`, Project: `",Project,"`, treated='",treated,"', control='",control,"', ",sep="\n")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "\n")    
    message1 <- paste("Factors= c(")
    write(message1, file = report,ncolumns = if(is.character(message1)) 1 else 5,append = TRUE, sep = "")   
    message2 <- paste("'",conditions[1:(length(conditions)-1)],"',",sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "")    
    message3 <- paste("'",conditions[length(conditions)],"'",sep="")
    write(message3, file = report,ncolumns = if(is.character(message3)) 1 else 5,append = TRUE, sep = "")    
    message4 <- paste("), ")
    write(message4, file = report,ncolumns = if(is.character(message4)) 1 else 5,append = TRUE, sep = "")    
    message1 <- paste("LibTypes= c(")
    write(message1, file = report,ncolumns = if(is.character(message1)) 1 else 5,append = TRUE, sep = "")    
    message2 <- paste("'",libTypes[1:(length(libTypes)-1)],"',",sep="")
    write(message2, file = report,ncolumns = if(is.character(message2)) 1 else 5,append = TRUE, sep = "")    
    message3 <- paste("'",libTypes[length(libTypes)],"'",sep="")
    write(message3, file = report,ncolumns = if(is.character(message3)) 1 else 5,append = TRUE, sep = "")    
    message4 <- paste("). ")
    write(message4, file = report,ncolumns = if(is.character(message4)) 1 else 5,append = TRUE, sep = "")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" This R code has been run:",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("  ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" ```{r} ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("require(DESeq2)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("the.file2='",the.file2,"'",sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("db <- InitDb(db.name=paste(the.file2,'deseq2_db',sep='_'), db.path='cache')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("x <- LoadCachedObject(db, 'maindataframe_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("the.file <- LoadCachedObject(db, 'the_file_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("Project <- LoadCachedObject(db, 'project_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("conditions <- LoadCachedObject(db, 'conditions_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("libtypes <- LoadCachedObject(db, 'libtypes_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("treated <- LoadCachedObject(db, 'treated_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("control <- LoadCachedObject(db, 'control_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("pvaladj <- LoadCachedObject(db, 'pvaladj_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("countData <- LoadCachedObject(db, 'countdata_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("  print('You loaded this count file: ')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("print(head(countData)) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("#colData = data.frame(row.names=colnames(countData), condition=conditions, libType=libTypes) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("colData <- LoadCachedObject(db, 'coldata_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("#dds <- DESeqDataSetFromMatrix(countData = countData, colData = colData, design = ~ condition) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'dds_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#colData(dds)$condition <- factor(colData(dds)$condition, levels=c(control,treated)) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'ddscond_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("######################################## ANALISYS ################################### ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#dds <- DESeq2::DESeq(dds) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("dds <- LoadCachedObject(db, 'ddsdes2_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#res <- results(dds) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("res <- LoadCachedObject(db, 'res_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")   
    message5 <- paste("print('Firts five lines of the results.') ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("print( head(res) ) ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("#list_DE_DESEQ2 = subset(as.data.frame(res), padj < pvaladj) # select significant genes",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("list_DE_DESEQ2 <- LoadCachedObject(db, 'listdedeseq2_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#PLOTS ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#ddsLocal <- estimateDispersions(dds, fitType='local')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n") 
    message5 <- paste("ddsLocal <- LoadCachedObject(db, 'ddsLocal_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("DESeq2::plotDispEsts(ddsLocal)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("#ddsMean <- estimateDispersions(dds, fitType='mean')",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("ddsMean <- LoadCachedObject(db, 'ddsmean_key')", sep="")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste("DESeq2::plotDispEsts(ddsMean)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
    message5 <- paste("DESeq2::plotDispEsts(dds)",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")    
    message8 <- paste(" ``` ",sep="\n")
    write(message8, file = report,ncolumns = if(is.character(message8)) 1 else 5,append = TRUE, sep = "\n")    
    message5 <- paste(" _______________________________________________________________________ ",sep="\n")
    write(message5, file = report,ncolumns = if(is.character(message5)) 1 else 5,append = TRUE, sep = "\n")
  }

  print(" FINISHED ! ")
  
  res

}
